package org.netkernel.layer0.representation;

/** HDS Hierarchical Data Structure
 * @author tab
 */
public interface IHDSNode
{
	/**
	 * Returns the name of the current node.
	 * @return Name of the node
	 */
	String getName();

	/**
	 * Returns the value stored in the current node.
	 * @return Value stored in the node
	 */
	Object getValue();
 
	/** Return all child nodes of this node
	*/
  	IHDSNode[] getChildren();

	/** Evaluate the given xpath from this node and return the list of matching nodes
	 * @param aXPath a simple HDS XPath expression
	 * @return a HDS node list
	 */
	IHDSNodeList getNodes(String aXPath);
	
	/** Return the first matching node or null
	 * @param aXPath a simple HDS XPath expression
	 * @return
	 */
	IHDSNode getFirstNode(String aXPath);
	
	/** Evaluate the given xpath from this node and return the list of values from matching nodes
	 * @param aXPath a simple HDS XPath expression
	 * @return
	 */
	Object[] getValues(String aXPath);
	
	/** Return the value of the first matching node or null
	 * @param aXPath a simple HDS XPath expression
	 * @return
	 */
	Object getFirstValue(String aXPath);

	/** return the root node of the hierarchy
	 */
  	IHDSNode getRoot();

	/** Return the parent node to this node, null if this is the root
	 */
	IHDSNode getParent();
  
	/** return the index position in the parent
	 */
	int getPositionInParent();
  
	/** return return the unique xpath for this node within the hierarchy
	 */
	String getXPath();
}