/******************************************************************************
 * (c) Copyright 2002-2006, 1060 Research Ltd
 *
 * This Software is licensed to You, the licensee, for use under the terms of
 * the 1060 Public License v1.0. Please read and agree to the 1060 Public
 * License v1.0 [www.1060research.com/license] before using or redistributing
 * this software.
 *
 * In summary the 1060 Public license has the following conditions.
 * A. You may use the Software free of charge provided you agree to the terms
 * laid out in the 1060 Public License v1.0
 * B. You are only permitted to use the Software with components or applications
 * that provide you with OSI Certified Open Source Code [www.opensource.org], or
 * for which licensing has been approved by 1060 Research Limited.
 * You may write your own software for execution by this Software provided any
 * distribution of your software with this Software complies with terms set out
 * in section 2 of the 1060 Public License v1.0
 * C. You may redistribute the Software provided you comply with the terms of
 * the 1060 Public License v1.0 and that no warranty is implied or given.
 * D. If you find you are unable to comply with this license you may seek to
 * obtain an alternative license from 1060 Research Limited by contacting
 * license@1060research.com or by visiting www.1060research.com
 *
 * NO WARRANTY:  THIS SOFTWARE IS NOT COVERED BY ANY WARRANTY. SEE 1060 PUBLIC
 * LICENSE V1.0 FOR DETAILS
 *
 * THIS COPYRIGHT NOTICE IS *NOT* THE 1060 PUBLIC LICENSE v1.0. PLEASE READ
 * THE DISTRIBUTED 1060_Public_License.txt OR www.1060research.com/license
 *
 * File:          $RCSfile$
 * Version:       $Name$ $Revision$
 * Last Modified: $Date$
 *****************************************************************************/
package org.netkernel.layer0.tools;

import java.net.FileNameMap;
import java.net.URLConnection;
import java.util.*;

/** Provides additional filename extention lookups to the JDK URLConnection classes
 * @author Tony Butterfield
 */
public class ExtraMimeTypes implements FileNameMap
{
	//TODO consider leveraging existing mime database such as freedesktop.org.xml
	// http://library.gnome.org/admin/system-admin-guide/stable/mimetypes-source-xml.html.en
	
	private static ExtraMimeTypes sInstance = new ExtraMimeTypes();
	
	static
	{	sInstance.mFNM=URLConnection.getFileNameMap();
		URLConnection.setFileNameMap(sInstance);		
	}
	
	public static ExtraMimeTypes getInstance()
	{	return sInstance;
	}
	
	private FileNameMap mFNM;
	private final Map mExtensionToMime;
	private final Map mMimeToParent;
	
	public ExtraMimeTypes()
	{	mExtensionToMime = new HashMap();
		mExtensionToMime.put("css","text/css");
		mExtensionToMime.put("js","application/x-javascript");
		mExtensionToMime.put("bsh","text/x-beanshell");
		mExtensionToMime.put("xsl","application/xslt+xml");
		mExtensionToMime.put("idoc","application/xml");
		mExtensionToMime.put("htc","text/x-component");
		
		mMimeToParent=new HashMap();
		mMimeToParent.put("image/svg+xml", "application/xml");
		mMimeToParent.put("application/xml", "text/xml");
	}
	
	public String getContentTypeFor(String fileName)
	{
		String result = mFNM.getContentTypeFor(fileName);
		if (result==null)
		{	int i=fileName.lastIndexOf('.');
			if (i>=0)
			{	String ext=fileName.substring(i+1);
				result = (String)mExtensionToMime.get(ext);
			}
		}
		return result;
	}
	
	
	/** Return true if aMimeA is assignable from aMimeB. I.e. aMimeB can be
	 * considered a specialisation of aMimeA
	 * @param aMimeA
	 * @param aMimeB
	 * @return
	 */
	public boolean isAssignableAFromB(String aMimeA, String aMimeB)
	{	String mime=aMimeB;
		while (mime!=null)
		{	if (mime.equals(aMimeA))
			{	return true;
			}
			mime=(String)mMimeToParent.get(mime);
		}
		return false;
	}
}