/******************************************************************************
 * (c) Copyright 2002,2003, 1060 Research Ltd
 *
 * This Software is licensed to You, the licensee, for use under the terms of
 * the 1060 Public License v1.0. Please read and agree to the 1060 Public
 * License v1.0 [www.1060research.com/license] before using or redistributing
 * this software.
 *
 * In summary the 1060 Public license has the following conditions.
 * A. You may use the Software free of charge provided you agree to the terms
 * laid out in the 1060 Public License v1.0
 * B. You are only permitted to use the Software with components or applications
 * that provide you with OSI Certified Open Source Code [www.opensource.org], or
 * for which licensing has been approved by 1060 Research Limited.
 * You may write your own software for execution by this Software provided any
 * distribution of your software with this Software complies with terms set out
 * in section 2 of the 1060 Public License v1.0
 * C. You may redistribute the Software provided you comply with the terms of
 * the 1060 Public License v1.0 and that no warranty is implied or given.
 * D. If you find you are unable to comply with this license you may seek to
 * obtain an alternative license from 1060 Research Limited by contacting
 * license@1060research.com or by visiting www.1060research.com
 *
 * NO WARRANTY:  THIS SOFTWARE IS NOT COVERED BY ANY WARRANTY. SEE 1060 PUBLIC
 * LICENSE V1.0 FOR DETAILS
 *
 * THIS COPYRIGHT NOTICE IS *NOT* THE 1060 PUBLIC LICENSE v1.0. PLEASE READ
 * THE DISTRIBUTED 1060_Public_License.txt OR www.1060research.com/license
 *
 * File:          $RCSfile$
 * Version:       $Name$ $Revision$
 * Last Modified: $Date$
 *****************************************************************************/
package org.netkernel.layer0.util;

/**
 * A low memory implementation of a multimap which preserves ordering. Implemented with an underlying array
 * and linear lookup time for get and remove.
 * @author  tab
 */
public final class OrderedCheapMultiMap
{
	/** the underlying list */
	private Object[] mItems;
	/** used index */
	private int mIndex;
	
	public static final OrderedCheapMultiMap EMPTY = new OrderedCheapMultiMap(0);
	
	/** Creates a new instance of OrderedCheapMultiMap */
	public OrderedCheapMultiMap(int aSize)
	{	mItems = new Object[aSize*2];
	}
	
	/** Clones a OrderedCheapMultiMap */
	public OrderedCheapMultiMap(OrderedCheapMultiMap aOther)
	{	mIndex= aOther.mIndex;
		mItems = new Object[mIndex];
		System.arraycopy(aOther.mItems, 0, mItems, 0, mIndex);
	}
	
	/** put a new key value pair in the collection */
	public void put(Object aKey, Object aValue)
	{	if (this==EMPTY)
		{	throw new UnsupportedOperationException();
		}
		if (mIndex==mItems.length)
		{	Object[] newItems = new Object[mItems.length*2];
			System.arraycopy(mItems, 0,newItems, 0, mItems.length);
			mItems=newItems;
		}
		mItems[mIndex++]=aKey;
		mItems[mIndex++]=aValue;
	}
	
	/** return the number of pairs in the collection */
	public int size()
	{	return mIndex>>1;
	}
	
	public int size(Object aKey)
	{	int result=0;
		for (int i=0; i<mIndex; i+=2)
		{	if (mItems[i].equals(aKey))
			{	result++;
			}
		}
		return result;
	}
	
	public boolean exists(Object aKey)
	{	boolean result=false;
		for (int i=0; i<mIndex; i+=2)
		{	if (mItems[i].equals(aKey))
			{	result=true;
				break;
			}
		}
		return result;
	}
	
	/** return the key value at the given index */
	public Object getKey(int aIndex)
	{	return mItems[aIndex*2];
	}
	
	/** return the value value at the given index */
	public Object getValue(int aIndex)
	{	return mItems[1+aIndex*2];
	}
	
	/** returns first value matching given key */
	public Object get(Object aKey)
	{	Object result = null;
		for (int i=0; i<mIndex; i+=2)
		{	Object key=mItems[i];
			if (key.equals(aKey))
			{	result=mItems[i+1];
				break;
			}
		}
		return result;
	}
	/** removes first value matching given key and returns it */
	public Object remove(String aKey)
	{	if (this==EMPTY)
		{	throw new UnsupportedOperationException();
		}
		Object result = null;
		for (int i=0; i<mIndex; i+=2)
		{	Object key=mItems[i];
			if (key.equals(aKey))
			{	result=mItems[i+1];
				mIndex-=2;
				System.arraycopy(mItems, i+2, mItems, i, mIndex-i);
				break;
			}
		}
		return result;
	}
	
	/** Replaces existing value with new one, old one is discarded
	 * @param aIndex
	 * @param aValue
	 */
	public void replaceValue(int aIndex, Object aValue)
	{	mItems[1+aIndex*2]=aValue;
	}
	
	public String toString()
	{	StringBuffer sb=new StringBuffer(256);
		sb.append('[');
		for (int i=0; i<mIndex; )
		{	if (i>0)
			{	sb.append(", ");
			}
			sb.append(mItems[i++].toString());
			sb.append("=\"");
			sb.append(mItems[i++].toString());
			sb.append('\"');
		}
		sb.append(']');
		return sb.toString();
	}
}